#--------------------------
# NicholsonBailey.R explores the Nicholson-Bailey Host-Parasitoid model
# to run, select each plot (code between the dashed lines) one at a time, copy, paste at the prompt, and hit enter
#--------------------------

#--------------------------
# functions and parameters: you need to enter this code before you do either plot

library(manipulate) # load library for user-manipulated plots

# function to run Nicholson-Bailey model
NB = function(tf, X0, params) {
	with(as.list(c(params,X0)), { # extract parameter values from vector
		X = matrix(0, nrow=tf, ncol=2, dimnames=list(c(),c("N","P"))) # create matrix with colums labeled for N and P
		X[1,] = c(N0, P0) # put in initial values
		for(t in 1:(tf-1)) { # loop through time steps
			X[t+1,"N"] = exp(r)*X[t,"N"]*exp(d*(-r*X[t,"N"]/K)-a*X[t,"P"]) # N_t+1
			X[t+1,"P"] = c*X[t,"N"]*(1-exp(-a*X[t,"P"])) # P_t+1
			X[t+1,] = pmax(X[t+1,],0) # don't allow to go negative
			}
			return(X)
		})
	}

# function to calculate equilibrium points for Nicholson-Bailey model without density dependence
NBeq = function(params) {
	with(as.list(params), { # for these parameters
		R = exp(r) # convert growth rate to growth factor
		Nbar = R*log(R)/(a*c*(R-1)) # calculate equilibrium values
		Pbar = log(R)/a
		Xbar=c(N0=Nbar, P0=Pbar)
		return(Xbar)
		})
	}

# parameters
r = 0.5 # population growth rate, where lambda=exp(r)
a = 0.001 # determines parasitoid attack rate
c = 3 # average number of parasitoids per host
tf1 = 50 # total run time
params = c(r=r, a=a, c=c, K=1000, d=0) # set parameters: d=0 indicates no density-dependence, so K is a dummy placeholder
Xbar = NBeq(params) # calculate equilibrium values without density dependence
X0 = c(N0=200, P0=100) # initial values
tf2 = 200 # run time
#------------------------------


#------------------------------
# First plot: host and parasitoind dynamics over time (no density dependence) given different initial conditions
manipulate({
  mf = par(mfrow=c(2,1),cex=0.75,mar = c(4,4,3,1)) # set up 2x1 subplot
  matplot(1:tf1, cbind(NB(tf1, diff*Xbar, params)[,"N"],NB(tf1, diff*Xbar, params)[,"P"]), 
          type="l", col=c("black","red"), xlab="Time", ylab="Abundance",main="Changing init. cond. (no dens. dep.)") 
  legend('topleft',col=c("black","red"),legend=c("Host","Parasitoid"),lty=1)
  matplot(1:tf2,cbind(NB(tf2, X0, c(r=r, a=a, c=c, K=K, d=1))[,"N"], NB(tf2, X0, c(r=r, a=a, c=c, K=K, d=1))[,"P"]),type="l", col=c("black","red"), xlab="Time", ylab="Abundance", main="Changing K (w/dens. dep.)")
  par(mfrow=mf)}, # finalize subplots
  diff = slider(0.95,1.05,step=0.01,initial=1),K = slider(250,2000,step=250,initial=250))
#------------------------------


