#--------------------------
# to run, select all, copy, paste at the prompt, and hit enter
#----IMPORTANT NOTE IF YOU'RE RUNNING THIS AT HOME:-------
# Numerically integrating continuous-time functions always
#  takes an extra "library" of R functions: deSolve
# To get this library, go to the menu for
#  Tools -> Install package(s)...
#  then search for deSolve, select it,
#  and click "Install"
# You only have to do this once.  
#--------------------------

library(manipulate) # load library for user-manipulated plots
library(deSolve) # loads the ODE solving package

#-------------------------
# functions

# Predator-prey model with density dependence in the prey
NPode = function(t, x, parms) {
  with(as.list(c(parms,x)), { # extract parameter values
    dN = r*N*(1-N/K)-b*N*P # prey
    dP = c*N*P-k*P # predator
    list(c(dN, dP)) # return
  })
}

# stochastic simulation
NPsims = function(K,r,k,b,c) {
  #r = 0.5 # prey birth rate
  #m = 0.2 # predator mortality rate
  #a = 0.01 # predation rate
  #b = 0.001 # predator birth rate from predation
  tf = 200 # run time
  N0 = floor(3*K/4) # initial value for prey
  P0 = floor(((r/b)*(1-k/(K*c))/2)) # initial value of predator
  dN = cbind(c(1,0),c(-1,0),c(-1,0),c(-1,1),c(0,-1)) # five possible outcomes: birth of prey, density-dependent mortality of prey, predation, predation w/birth of predator, death of predator
  nOut = ncol(dN) # number of possible outcomes
  # initialize
  tstep = 1; vt = 0; # times
  NP = as.matrix(c(N0,P0)); # initial population sizes
  # stochastic simulation
  while(vt[tstep] < tf) { # run until hit tf
    wt = c(r*NP[1,tstep],r*NP[1,tstep]^2/K,(b-c)*NP[1,tstep]*NP[2,tstep],c*NP[1,tstep]*NP[2,tstep],k*NP[2,tstep]) # wait time for each event - make sure that this is in the same order as dN
    sum_wt = sum(wt) # total rate
    wtn = wt/sum_wt # normalized rates
    dt = rexp(n=1, rate=sum_wt) # calculate wait time until next event
    outIdx = sample(1:nOut, size=1, prob=wtn) # determine what next event is
    NP = cbind(NP, NP[,tstep]+dN[,outIdx]) # change population sizes according to next outcome
    vt[tstep+1] = vt[tstep]+dt # update time step
    tstep = tstep+1 # update step tracker
  }
  # deterministic simulation
  parms = c(r=r,k=k,b=b,c=c,K=K) # parameter vector
  x0 = c(N=N0,P=P0) # initial values vector
  out = as.data.frame(lsoda(x0, vt, NPode, parms)) # run ODE
  return(cbind(vt,NP[1,],NP[2,],out$N,out$P))
}

#-------------------------

# run and plot results
manipulate({
  mf = par(mfrow=c(2,1)) # set up 2x1 subplot
  NPsimsout = NPsims(K,r,k,b,c) # run simulations
  matplot(NPsims(K,r,k,b,c)[,c(2,4)], 
          type="l", col=c(1,2), lty=c(1,2), xlab="Time (number of steps; total run time is 200 years)", ylab="Prey abundance")
  matplot(NPsims(K,r,k,b,c)[,c(3,5)], 
          type="l", col=c(1,2), lty=c(1,2), xlab="Time (number of steps; total run time is 200 years)", ylab="Predator abundance")
  par(mfrow=mf) # finalize subplots
},
  K=slider(500,1500,step=100,initial=1000),
  r=slider(0.2,1,step=0.1,initial=0.5),
  k=slider(0.15,0.4,step=0.05,initial=0.2),
  b=slider(0.001,0.015,step=0.001,initial=0.01),
  c=slider(0.0005,0.002,step=0.0001,initial=0.001))


