#--------------------------
# to run, select all, copy, paste at the prompt, and hit enter
#----IMPORTANT NOTE IF YOU'RE RUNNING THIS AT HOME:-------
# Numerically integrating continuous-time functions always
#  takes an extra "library" of R functions: deSolve
# To get this library, go to the menu for
#  Tools -> Install package(s)...
#  then search for deSolve, select it,
#  and click "Install"
# You only have to do this once.  
#--------------------------

library(manipulate) # load library for user-manipulated plots
library(deSolve) # loads the ODE solving package

#-------------------------
# functions

# SI model with births and deaths for deterministic integration
SIbd = function(t, x, parms) {
	with(as.list(c(parms,x)), { # extract parameter values
		dS = p*(N-S)-R0*S*I/N # susceptibles
		dI = R0*S*I/N-I # infecteds
		list(c(dS, dI)) # return dS and dI
		})
	}

# stochastic simulation
SIRsims = function(N,p,R0) {
  # parameters and initial conditions
  alpha = 1-p # recovery rate
  tf = 100 # run time
  N0 = N # store value for total population size
  I0 = floor(0.01*N) # initial value for infecteds
  S0 = floor(0.4*N) # initial value of susceptibles
  Rec0 = N-S0-I0 # initial value for recovereds
  dN = cbind(c(1,0,0), c(-1,0,0), c(-1,1,0), c(0,-1,0), c(0,-1,1), c(0,0,-1)) # six possible outcomes: birth, death of susceptible, infection, death of infected, recovery of infected, death of recovered
  nOut = ncol(dN) # number of possible outcomes
  # initialize
  tstep = 1; vt = 0; # times
  SIR = as.matrix(c(S0, I0, Rec0)); # initial population sizes
  # stochastic simulation
  while(vt[tstep] < tf) { # run until hit tf
    N = sum(SIR[,tstep]) # total population size
    wt = c(p*N, p*SIR[1,tstep], R0*SIR[1,tstep]*SIR[2,tstep]/N, p*SIR[2,tstep], alpha*SIR[2,tstep], p*SIR[3,tstep]) # rate for each event - make sure that this is in the same order as dN
    sum_wt = sum(wt) # total rate
    wtn = wt/sum_wt # normalized rates
    dt = rexp(n=1, rate=sum_wt) # calculate wait time until next event
    outIdx = sample(1:nOut, size=1, prob=wtn) # determine what next event is
    SIR = cbind(SIR, SIR[,tstep]+dN[,outIdx]) # change population sizes according to next outcome
    vt[tstep+1] = vt[tstep]+dt # update time step
    tstep = tstep+1 # update step tracker
  }
  # deterministic simulation
  parms = c(R0=R0, p=p, N=N0) # parameter vector
  x0 = c(S=S0, I=I0) # initial values vector
  out = as.data.frame(lsoda(x0, vt, SIbd, parms)) # run ODE
  return(cbind(vt,SIR[1,],SIR[2,],out$S,out$I))
}

#-------------------------

# run and plot results
manipulate(
  matplot(SIRsims(N,p,R0)[,-1], 
          type="l", col=c(1,2,1,2), lty=c(1,1,2,2), xlab="Time (number of steps; total run time is 100 years)", ylab="Abundance",main="Susceptibles in black,\n infecteds in red"),
  N=slider(250,2000,step=250,initial=1000),
  R0=slider(2,16,step=2,initial=8),
  p=slider(0.01,0.08,step=0.01,initial=0.04))
