#--------------------------
# No plot manipulation, but be sure to note the values outputed to the command line
# BE SURE TO DOWNLOAD THE DATA FILES parameciumrk.txt and paramecium2.txt
# and CHANGE THE DIRECTORY (Session menu, Set Working Directory) 
# to where you've downloaded these files
# in order for the script to run
#----IMPORTANT NOTE IF YOU'RE RUNNING THIS AT HOME:-------
# Numerically integrating continuous-time functions always
#  takes an extra "library" of R functions: deSolve
# To get this library, go to the menu for
#  Tools -> Install package(s)...
#  then search for deSolve, select it,
#  and click "Install"
# You only have to do this once.  
#--------------------------
# to run, select all, copy, paste at the prompt, and hit enter
#--------------------------

library(deSolve) # loads the ODE numerical integration package

#----------------------------------
# function for numerically integrating the competition model
lvcomp = function(t, n, parms) {
	with(as.list(parms), { # extract parameters from parms vector
		dn = rep(0, 2) # initialize dn/dt vector
		dn[1] = r1*n[1]*(K1-n[1]-a1*n[2])/K1 # species 1
		dn[2] = r2*n[2]*(K2-n[2]-a2*n[1])/K2 # species 2
		return(list(dn)) # return dn/dt as a list
		})
	}

# function to minimize for fitting data to competition model
LVcompMin = function(a) {
	parms = c(parmsrk, a=a) # add gueses for alphas to parameters
	out = as.data.frame(lsoda(n0, times, lvcomp, parms)) # run ode
	mse = mean((rbind(out$n1, out$n2)-t(nTrue))^2) # calculate mean squared error between simulated and original data
	return(mse) # return mean squared error
	}


#---------------------------------
# Parameters:
# these r and K values come from Paramecium experiments by 
# Gause (1935) - fit to logistic growth model when each species
# is on its own, without competition
rk = as.matrix(read.table("parameciumrk.txt")) # load data for r and K
colnames(rk) = NULL # remove column names
parmsrk = c(r=rk[1,], K=rk[2,]) # store r and K in labeled parameters vector
# Paramecium data from Gause 1935 - we'll find the alpha's that best fit this data
nTrue = as.matrix(read.table("paramecium2.txt")) # load time series data of species in competition
colnames(nTrue) = NULL # remove column names
# Time for simulations
tf = nrow(nTrue) # end time
times = 1:tf # vector of times to run over - same as that for data

a0 = rep(0, ncol(nTrue)) # initial guess for [a12, a21]: no competition
n0 = c(n=nTrue[1,]) # initial values - from data, first column

# find fit and run simulations
optimOut = optim(a0, LVcompMin) # find best fit a to LVcompMin given initial guess a0
a = optimOut$par # extract answer that minimizes LVcompMin
parms = c(parmsrk, a=a) # add alpha's to parameter vector
nSim = as.data.frame(lsoda(n0, times, lvcomp, parms)) # run ode to get simulated data for the best-fit alphas

# plot results
plot(times, nTrue[,1], col="red", type="p", xlab="Time", ylab="Abundance", main="Paramecium aurelia and P. caudatum  in competition") # plot points for actual data - red for first species
points(times, nTrue[,2], col="blue") #  and blue for second species
lines(times, nSim$n1, col="red") # plot simulations given best-fit parameters - red for first species
lines(times, nSim$n2, col="blue") #  and blue for second species
legend("topleft",legend=c("Species 1","Species 2"),lty=1,col=c("red","blue")) # add a legend

# report best fit parameter values
print(parmsrk)
print("alpha12, alpha21 =")
print(a)
