#------
# libraries

library(manipulate) # load library for user-manipulated plots
library(deSolve) # loads the ODE numerical integration package

#--------------------------
# functions

# This code is based on the constant-effort harvest model:
# dn/dt = rn(1-n/K)-qen

# equilibrium population size
eqPopSize = function(r,K,h) {
  Nbar = K*(1-h/r) # internal equilibrium
  Nbar = (Nbar>0)*Nbar # if the internal equilibrium is not positive, then the locally stable equilibrium is n=0
  return(Nbar)
}

# equilibrium yeild
eqYield = function(r,K,h) {
  Ybar = h*eqPopSize(r,K,h) # harvest rate * equilibrium population size
  return(Ybar)
}

# ODE to integrate for time series
dndt = function(t, n, parms) {
  with(as.list(parms), { # extract parameters from parms vector
    dn = r*n*(1-n/K)-h*n # mode to integrate
    return(list(dn)) # return dn/dt as a list
  })
}


#--------------------------
# parameters
rmax = 1.5 # maximum value for population growth rate
hmax = rmax+0.1 # maximum value for harvest rate
Kmax = 1000 # maximum carrying capacity
h = seq(0,hmax,by=0.01) # x-axis for plots: vector of harvest rates
times = 1:20 # times to integrate over
n0 = c(n=seq(100,1.5*Kmax,100)) # initial conditions for integration over time
Nn0 = length(n0) # number of initial conditions to integrate over

#--------------------------
# plots
manipulate({
  nt1 = as.data.frame(lsoda(n0, times, dndt, c(r=r,K=K,h=h1))) # run ode for first harvest value to get the poulation size over time
  nt2 = as.data.frame(lsoda(n0, times, dndt, c(r=r,K=K,h=h2))) # run ode for second harvest value to get the poulation size over time
  mf = par(mfrow=c(2,2)) # set up 2x1 subplot
  plot(h,eqPopSize(r,K,h),type="l",col="blue",lwd=2,xlab="Harvest rate (q*e)",ylab="Equilibrium population size") # plot equilibrium population size
  plot(h,eqYield(r,K,h),type="l",col="blue",lwd=2,,xlab="Harvest rate (q*e)",ylab="Equilibrium yield") # plot equilibrium yeild
  matplot(times,nt1[,2:(Nn0+1)],type="l",xlab="Time",ylab="Population size",ylim=c(0,Kmax),main=paste("Harvest rate = ",h1)) # plot time series for the first harvest value
  matplot(times,nt2[,2:(Nn0+1)],type="l",xlab="Time",ylab="Population size",ylim=c(0,Kmax),main=paste("Harvest rate = ",h2)) # plot time series for the second harvest value
  par(mfrow=mf) # finalize subplots
},
r=slider(0.1,rmax,step=0.1,initial=0.8),K=slider(100,Kmax,step=100,initial=900),h1=slider(0,hmax,step=0.1,initial=0.1),h2=slider(0,hmax,step=0.1,initial=hmax))
