#--------------------------
# calculates stable age structure and fecundities give vole data for lx and mx
# to run, select all, copy, paste at the prompt, and hit enter
#--------------------------

library(manipulate) # load library for manipulating the plot

# default values - vole data
lxdef = c(1, 0.8445, 0.7313, 0.5881, 0.4334, 0.2928, 0.1813, 0.1029, 0.0535, 0.0255) # survivorships at each age
mxdef = c(0, 0.6504, 2.3939, 2.9727, 2.4662, 1.7043, 1.0815, 0.6683, 0.4286, 0.3000) # fecundities at each age
xdef = 0:(length(lxdef)-1) # vector of ages

# to find the intrinsic growth rate, we'll need to solve the Euler-Lotka (1=sum(lx*mx*exp(-r*(x-1)))) equation for r
# R has a function that will find the minimum of any function, so we can use this and the fact that (sum(lx*mx*exp(-r*(x-1)))-1)^2 should be minimized (=0) when r is the solution to the Euler-Lotka equation; we use the square instead of the absolute value because it's smoother at the minimum, which makes it easier for R to find the minimum
# so, to plug into R's minimum-finding function, we'll have to create a function that calculates (sum(lx*mx*exp(-r*(x-1)))-1)^2
ELmin = function(r,lx,mx,x) {
	# function to minimize to solve Euler-Lotka equation
	EL = sum(lx*mx*exp(-r*x)) # Euler-Lotka equation
	ssd = sum((EL-1)^2) # sum of squared deviations
	return(ssd) # return squared deviations
	}
	
# function to find r
findr = function(lx,mx,x) {
  rspan = 20 # range to search for r over, pick something large where you're sure r will be in there
  rfit = optimize(function(r)ELmin(r,lx,mx,x), c(-rspan, rspan)) # run R optimization function over this range for ELmin
  r = rfit$minimum # extract value that minimizes ELmin from R's answer
  return(r)
}

# function to find R0
findR0 = function(lx,mx) sum(lx*mx)

# funciton to find stable age distribution
findcx = function(lx,mx,x) {
  r = findr(lx,mx,x)
  cx = lx*exp(-r*x) 
  cx = cx/sum(cx) # normalized stable age distribution
  return(cx)
}

# function to find reproductive values
findvx = function(lx,mx,x) {
  r = findr(lx,mx,x)
  rev(cumsum(rev(mx)*rev(lx)*exp(-r*rev(x))))/(exp(-r*x)*lx)
}

# plot
manipulate(
  barplot(plotFn(lxcoeff*lxdef,mxcoeff*mxdef*(xdef>=xmat),xdef), 
          xlab="Age class", ylab="Relative value", beside=TRUE, names.arg=xdef, main=paste("R0=",round(findR0(lxcoeff*lxdef,mxcoeff*mxdef*(xdef>xmat)),digits=2),", r=",round(findr(lxcoeff*lxdef,mxcoeff*mxdef*(xdef>xmat),xdef),digits=2),sep="")),
  lxcoeff=slider(0.1,1,step=0.1,initial=1),
  mxcoeff=slider(0.1,1,step=0.1,initial=1),
  xmat=slider(1,6,step=1,initial=1),
  plotFn = picker("Age distribution"=findcx,"Reproductive values"=findvx))
