#--------------------------
# put in values here
vr = c() # prey population growth rate: values to explore -- enter a few values, separated by commas
vK = c() # prey carrying capacity: values to explore -- enter a few values, separated by commas
rdef = 0.5 # prey population growth rate: default (value when varying K)
Kdef = 1000 # prey carrying capacity: default (value when varying r)
svplt = FALSE # TRUE for saving plots to a pdf file, FALSE for displaying in R
#----IMPORTANT NOTE:-------
# Numerically integrating continuous-time functions always
#  takes an extra "library" of R functions: deSolve
# To get this library, go to the menu for
#  PC: Packages -> Install package(s)...
#  Mac: Packages & Data -> Package Installer
#  then search for deSolve, select it,
#  and click "Install Selected"
# You only have to do this once.  
#--------------------------
# to run, either
# select all, copy, paste at the prompt, and hit enter
# - or -
# change the directory (File menu on PC, Misc menu on Mac) to where the file is, and type "source("predPrey.R")" at the prompt
#--------------------------

library(deSolve) # loads the ODE numerical integration package

# Function for the Lotka-Volterra predator-prey model: everything linear
PredPreyLin = function(t, n, parms) {
	H = n[1] # prey are first species
	P = n[2] # predators are second species
	with(as.list(parms), { # extract parameters from vector parms
		dHdt = r*H-b*H*P # prey (dH/dt)
		dPdt = c*H*P-k*P # predator (dP/dt)
		list(c(dHdt, dPdt)) # return dn/dt
		})
	}

# Function for the Lotka-Volterra predator-prey model: density dependence in prey
PredPreyDDI = function(t, n, parms) {
	H = n[1] # prey are first species
	P = n[2] # predators are second species
	with(as.list(parms), { # extract parameters from vector parms
		dHdt = r*H*(1-H/K)-b*H*P # prey (dH/dt)
		dPdt = c*H*P-k*P # predator (dP/dt)
		list(c(dHdt, dPdt)) # return dn/dt
		})
	}

# Function for the Lotka-Volterra predator-prey model: density dependence in prey, type II functional response
PredPreyDDII = function(t, n, parms) {
	H = n[1] # prey are first species
	P = n[2] # predators are second species
	with(as.list(parms), { # extract parameters from vector parms
		dHdt = r*H*(1-H/K)-b*H*P/(1+d*H) # prey (dH/dt)
		dPdt = c*H*P/(1+d*H)-k*P # predator (dP/dt)
		list(c(dHdt, dPdt)) # return dn/dt
		})
	}

# Function for the Lotka-Volterra predator-prey model: density dependence in prey, type III functional response
PredPreyDDIII = function(t, n, parms) {
	H = n[1] # prey are first species
	P = n[2] # predators are second species
	with(as.list(parms), { # extract parameters from vector parms
		dHdt = r*H*(1-H/K)-b*H^2*P/(1+d*H^2) # prey (dH/dt)
		dPdt = c*H^2*P/(1+d*H^2)-k*P # predator (dP/dt)
		list(c(dHdt, dPdt)) # return dn/dt
		})
	}


# Parameters
b = 0.01 # predator attack rate
c = 0.1*b # predator conversion of preation into reproduction
k = 0.2 # predator mortality
a = 1 # rate of prey capture/unit prey and time
d = 0.0015 # handling time

# Time
tf = 200 # final time
times = seq(1,tf,by=0.2) # vector of times

# Initial conditions: three different choices
n0 = c(H=300, P=50)
nmods = 4 # how may different models to simulate (linear, density dependence, type II, type III)

# run model and plot
for(steprK in 1:2) { # loop through whether switching r or K
	if(svplt) pdf(file=paste("predPrey",switch(steprK,"r","K"),".pdf",sep="")) else dev.new() # create file if saving plot to pdf, otherwise open new plot window
	mf = par(mfrow=c(nmods,3)) # set up subplots - pair with par(mfrow=mf) below
	nruns = length(switch(steprK,vr,vK)) # number of runs
	for(stepMod in 1:nmods) {# step through each model
		Heq = rep(0,nruns) # initialize vectors for equilibria
		Peq = Heq
		Ht = matrix(0,nrow=length(times),ncol=nruns) # initialize matrices for time series
		Pt = Ht
		for(stepVal in 1:nruns) { # loop for values of r or K
			if(steprK==1) { # set r and K values
				r = vr[stepVal] # set r to value for this iteration
				K = Kdef # set K to default
				} else {
				r = rdef # set r to default
				K = vK[stepVal] # set K to value for this iteratiohn
				}
			parms = c(r=r, b=b, c=c, k=k, K=K, d=d) # set parameter vector
			if(stepMod==1) { # equilibrium values, model to run depend on which model running
				Heq[stepVal] = k/c # calculate equilibrium values
				Peq[stepVal] = r/b
				out = as.data.frame(lsoda(n0, times, PredPreyLin, parms)) # run model
				} else if(stepMod==2) {
				Heq[stepVal] = k/c # calculate equilibrium values
				Peq[stepVal] = (r/b)*(1-k/(K*c))
				out = as.data.frame(lsoda(n0, times, PredPreyDDI, parms)) # run model
				} else if(stepMod==3) {
				Heq[stepVal] = k/(c-k*d) # calculate equilibrium values
				Peq[stepVal] = (r/b)*(1-Heq[stepVal]/K)*(1+d*Heq[stepVal])
				out = as.data.frame(lsoda(n0, times, PredPreyDDII, parms)) # run model
				} else {
				Heq[stepVal] = sqrt(k/(c-k*d)) # calculate equilibrium values
				Peq[stepVal] = (r/b)*(1+d*(Heq[stepVal])^2)*(1/Heq[stepVal]-1/K)
				out = as.data.frame(lsoda(n0, times, PredPreyDDIII, parms)) # run model
				}
			# extract output
			Ht[,stepVal] = out$H
			Pt[,stepVal] = out$P
			}
		# plot results
		matplot(Ht,Pt,type="l",lty=1,col=1:3,xlab="Prey",ylab="Predator") # phase-plane plot of predator vs. prey
		points(Ht[1,],Pt[1,],pch=21,col=1:3) # mark starting points with open circles
		points(Heq,Peq,pch=8,col=1:3) # mark equilibria with asteriks
		matplot(times,Ht,type="l",lty=1,col=1:3,xlab="Time",ylab="Prey",main=switch(stepMod,"Linear","Type I w/DD in prey","Type II w/DD in prey","Type III w/DD in prey")) # prey vs. time
		matplot(times,Pt,type="l",lty=1,col=1:3,xlab="Time",ylab="Predator") # predator vs. time
		}
	legend("topright",paste(switch(steprK,"r","K"),"=",switch(steprK,vr,vK)),lty=1,col=1:3) # add a legend
	par(mfrow=mf) # subplots
	if(svplt) dev.off() # shut down current plot for saving to pdf
	}


