#--------------------------
# to run, select all, copy, paste at the prompt, and hit enter
#----IMPORTANT NOTE IF YOU'RE RUNNING THIS AT HOME:-------
# Numerically integrating continuous-time functions always
#  takes an extra "library" of R functions: deSolve
# To get this library, go to the menu for
#  Tools -> Install package(s)...
#  then search for deSolve, select it,
#  and click "Install"
# You only have to do this once.  
#--------------------------

library(manipulate) # load library for user-manipulated plots
library(deSolve) # loads the ODE solving package

#-------------------------
# functions

# Lotka-Volterra competition model with rarefaction
LVcompode = function(t, n, parms) {
  with(as.list(c(parms)), { # extract parameter values
    dn = c(0,0)
    dn[1] = r1*n[1]*(K1-n[1]-a1*n[2])/K1-m*n[1] # species 1
    dn[2] = r2*n[2]*(K2-n[2]-a2*n[1])/K2-m*n[2] # species 2
    list(dn) # return
  })
}

# Plot simulations and isoclines for different rarefaction values
plotLVcomp = function(m) {
  parms = c(r=c(0.5,1.5),K=c(100,100),a=c(0.5,1.2),m=m) # parameters: growth rates r, carrying capacities K, competitive effects a, and rarefaction rate m
  # plot isoclines
  vn1 = 0:parms["K1"] # vector of values for n1 to plot over
  n1iso = with(as.list(parms),(K1/(r1*a1))*((r1/K1)*(K1-vn1)-m)) # species 1 isocline (n2 vs. n1 when dn1/dt=0)
  n2iso = with(as.list(parms),(K2/r2)*((r2/K2)*(K2-a2*vn1)-m)) # species 2 isocline (n2 vs. n1 when dn2/dt=0)
  matplot(vn1,cbind(n1iso,n2iso),col=c('red','blue'),type="l",lwd=2,lty=1,xlab='n_1',ylab='n_2',xlim=c(0,parms["K1"]),ylim=c(0,parms["K2"])) # plot isoclines
  # plot simulations
  nstep = 10 # step size for initial conditions
  vn10 = seq(nstep,parms["K1"],by=nstep) # initial condition values for n1
  vn20 = seq(nstep,parms["K2"],by=nstep) # initial condition values for n2
  tf = 100 # final time
  for(step1 in 1:length(vn10)) {
    for(step2 in 1:length(vn20)) {
      n0 = c(n=c(vn10[step1],vn20[step2]))
      nSim = as.data.frame(lsoda(n0, 1:tf, LVcompode, parms)) # run ode to get simulated data for that a
      lines(nSim$n1,nSim$n2,lty=1,col='black') # plot simulation output
      points(n0[1],n0[2],pch='o',cex=0.5,col='black') # mark starting point
    }
  }
  points(nSim$n1[tf],nSim$n2[tf],pch='*',cex=4,col='green')
}

#-------------------------
# run and plot results
manipulate(plotLVcomp(m),m=slider(0,step=0.05,0.25,initial=0))
